// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

mixin DisposableAttachment {
  void dispose() {}
}

mixin AttachmentMixin {
  Map? _attachments;

  void addAttachment(Object key, dynamic attachment) {
    _attachments ??= {};
    _attachments![key] = attachment;
  }

  T? getAttachment<T>(Object key) {
    return _attachments?[key];
  }

  T ensureAttachment<T>(Object key) {
    return _attachments?[key];
  }

  void removeAttachment(Object key) {
    final attachment = _attachments?.remove(key);
    _disposeAttachment(attachment);
  }

  void clearAttachments() {
    final attachments = _attachments?.values.toList() ?? [];
    _attachments?.clear();
    attachments.forEach((attachment) {
      _disposeAttachment(attachment);
    });
  }

  void _disposeAttachment(dynamic attachment) {
    if (attachment is DisposableAttachment) {
      try {
        attachment.dispose();
      } catch (e, s) {
        if (kDebugMode) {
          debugPrint('dispose attachment error: $e\n$s');
          throw e;
        }
      }
    }
  }
}
